@@ -19,7 +19,6 @@ import six
 import cheroot.server
 import cheroot.wsgi
 
-from cheroot._compat import bton
 from cheroot.test import webtest
 
 log = logging.getLogger(__name__)
@@ -158,16 +157,10 @@ class Controller(object):
     def __call__(self, environ, start_response):
         req, resp = Request(environ), Response()
         try:
-            # Recode handler name as Python supports unicode method names
-            # and we should try match those first.
-            # Source of the idea: cherrypy._cpwsgi.AppResponse.recode_path_qs
-            latin1_handler_name = environ['PATH_INFO'].lstrip('/').replace('/', '_')
-            try:
-                unicode_handler_name = bton(latin1_handler_name.encode('iso-8859-1'), 'utf-8')
-                handler = getattr(self, unicode_handler_name)
-            except (UnicodeError, AttributeError):
-                handler = getattr(self, latin1_handler_name)
-        except AttributeError:
+            # Python 3 supports unicode attribute names
+            # Python 2 encodes them
+            handler = self.handlers[environ['PATH_INFO']]
+        except KeyError:
             resp.status = '404 Not Found'
         else:
             output = handler(req, resp)
