@@ -25,9 +25,11 @@ import json
 import unittest
 from imp import reload
 
-from six.moves import range, http_client, map
+from six.moves import range, http_client, map, urllib_parse
 import six
 
+from more_itertools.more import always_iterable
+
 
 def interface(host):
     """Return an IP address for a client connection given the server host.
@@ -258,6 +260,21 @@ class WebCase(unittest.TestCase):
                 protocol=None, raise_subcls=None):
         """Open the url with debugging support. Return status, headers, body.
 
+        url should be the identifier passed to the server, typically a
+        server-absolute path and query string (sent between method and
+        protocol), and should only be an absolute URI if proxy support is
+        enabled in the server.
+
+        If the application under test generates absolute URIs, be sure
+        to wrap them first with strip_netloc::
+
+            class MyAppWebCase(WebCase):
+                def getPage(url, *args, **kwargs):
+                    super(MyAppWebCase, self).getPage(
+                        cheroot.test.webtest.strip_netloc(url),
+                        *args, **kwargs
+                    )
+
         `raise_subcls` must be a tuple with the exceptions classes
         or a single exception class that are not going to be considered
         a socket.error regardless that they were are subclass of a
@@ -586,6 +603,29 @@ def openURL(url, headers=None, method='GET', body=None,
                     raise
 
 
+def strip_netloc(url):
+    """
+    Strip the scheme and host from the URL, returning the
+    server-absolute portion.
+
+    Useful for wrapping an absolute-URI for which only the
+    path is expected (such as in calls to getPage).
+
+    >>> strip_netloc('https://google.com/foo/bar?bing#baz')
+    '/foo/bar?bing'
+
+    >>> strip_netloc('//google.com/foo/bar?bing#baz')
+    '/foo/bar?bing'
+
+    >>> strip_netloc('/foo/bar?bing#baz')
+    '/foo/bar?bing'
+    """
+    parsed = urllib_parse.urlparse(url)
+    scheme, netloc, path, params, query, fragment = parsed
+    stripped = '', '', path, params, query, ''
+    return urllib_parse.urlunparse(stripped)
+
+
 # Add any exceptions which your web framework handles
 # normally (that you don't want server_error to trap).
 ignored_exceptions = []
@@ -617,22 +657,3 @@ def server_error(exc=None):
         print('')
         print(''.join(traceback.format_exception(*exc)))
         return True
-
-
-def always_iterable(obj, base_type=(six.text_type, six.binary_type)):
-    """
-    Copy of always_iterable from more_itertools.more.
-
-    Replace with dependency when Python 2.6 compatibility is
-    no longer required.
-    """
-    if obj is None:
-        return iter(())
-
-    if (base_type is not None) and isinstance(obj, base_type):
-        return iter((obj,))
-
-    try:
-        return iter(obj)
-    except TypeError:
-        return iter((obj,))
